/* 
 * File:   main.c
 * Author: Pavel Dvorak
 * Created on 15. leden 2020, 18:39
 * 
 * MPLAB X IDE v.4.20 XC8 1.42
 * long int 32 bit
 */

#include <xc.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "oled_i2c.h"
#include <ctype.h>



#pragma config FOSC = INTOSC    // Oscillator Selection->INTOSC oscillator: I/O function on CLKIN pin
#pragma config WDTE = OFF    // Watchdog Timer Enable->WDT disabled
#pragma config PWRTE = OFF    // Power-up Timer Enable->PWRT disabled
#pragma config MCLRE = ON    // MCLR Pin Function Select->MCLR/VPP pin function is MCLR
#pragma config CP = OFF    // Flash Program Memory Code Protection->Program memory code protection is disabled
#pragma config CPD = OFF    // Data Memory Code Protection->Data memory code protection is disabled
#pragma config BOREN = ON    // Brown-out Reset Enable->Brown-out Reset enabled
#pragma config CLKOUTEN = OFF    // Clock Out Enable->CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
#pragma config IESO = ON    // Internal/External Switchover->Internal/External Switchover mode is enabled
#pragma config FCMEN = ON    // Fail-Safe Clock Monitor Enable->Fail-Safe Clock Monitor is enabled

// CONFIG2
#pragma config WRT = OFF    // Flash Memory Self-Write Protection->Write protection off
#pragma config VCAPEN = ON    // Voltage Regulator Capacitor Enable bit->Vcap functionality is disabled on RA6.
#pragma config PLLEN = OFF    // PLL Enable->4x PLL disabled
#pragma config STVREN = ON    // Stack Overflow/Underflow Reset Enable->Stack Overflow or Underflow will cause a Reset
#pragma config BORV = LO    // Brown-out Reset Voltage Selection->Brown-out Reset Voltage (Vbor), low trip point selected.
#pragma config LPBOR = OFF    // Low Power Brown-Out Reset Enable Bit->Low power brown-out is disabled
#pragma config LVP = ON    // Low-Voltage Programming Enable->High-voltage on MCLR/VPP must be used for programming

char addr = 0x78; //SSD1306_ADDRESS	0x3C
int tup;
long nap;
char uadc;
int i; 
char len;
char fre = 1, tlc = 0, menu = 0;
char start = 2;
char rng = 1;
char irng = 0;
char mpx = 5;
char vmx = 1;
char vmr = 1;
char over = 0;
char bgr = 0;
char j=0; 
char led = 0;
char port;
long volt;
int dack = 522;
int avolt;
int mhi, mlo;
long offs ;
char str[20]=" ";
char strt[20]=" ";


// tabulka nastaven vstupnho dlie
char Rng[8][2]={
    {5,1},  // 0
    {0,0},   //1    2M      2V
    {1,1},  // 2    1M      4V
    {2,2},  // 3    500k    8V
    {3,4},  // 4    250k    16V
    {4,8},  // 5    125k    32V
    {6,16}, // 6    62,5 k  64V
    {7,32}  // 7    31,25 k 128V 
};

// tabulka pro nastaven proudu 
int Irng[8][2]={
    {1,   77},   // 0    0.5mA
    {2,  528},   // 1    0.25
    {4,  750},   // 2    0.125
    {8,  861},   // 3    0.0625
    {16, 917},   // 4    0.03125
    {32, 945},   // 5    0.015625
    {64, 959},   // 6    0.0078125
    {1,  999}    // 7
};

void OpenI2C(char sync_mode, char slew) {
    SSPSTAT &= 0x3F; // power on state
    SSPCON = 0x00; // power on state                // power on state
    SSPCON |= sync_mode; // select serial mode
    if (slew)SMP = 1; // slew rate  0 = high speed 400kHz, 1 = standard 100kHz nebo 1MHz
    SSPEN = 1; // enable synchronous serial port
}//

void StartI2C() {
    SEN = 1;
    while (SEN);
}//


void StopI2C() {
    PEN = 1;
    while (PEN);
}//


void Sdelay(char dl){  // 7us
    while(dl){
    _delay(127);
    dl--;
    }
}//

void Delay(unsigned int tms) { //ms
    while (tms) {
        _delay(8000);
        tms--;
    }
}//



void WriteI2C(char ch){
        SSP1IF=0;
        SSPBUF=ch;      
     while(!SSP1IF);          
}//

void Dac(int i){
    nap = 0;  
    ADON = 1;
    Sdelay(2);
    GO = 1;
    while (GO);
    for(i; i>0; i--)
    {   
    Sdelay(15);
    GO = 1;
    while (GO);    
    tup = ADRESL;
    uadc = ADRESH;        
   *((char *) &tup + 1) = uadc;
   nap += tup; 
  }      
 ADON = 0;
} //Dac

void Beep(char ton){

if (ton==0){TMR2IE=0; PORTBbits.RB0 = 0; } 
else {
T2CONbits.T2OUTPS=ton-1; TMR2IE=1;
}  
       
} //

void Leds(char led)
{ 
switch(led){
    case 0: TRISCbits.TRISC6 = 1;
    TRISAbits.TRISA7 = 1;
    break;
    case 1: TRISCbits.TRISC6 = 0;
    PORTCbits.RC6= 1;
    TRISAbits.TRISA7 = 0;
    PORTAbits.RA7 = 0;
    break;
    case 2: TRISAbits.TRISA7 = 0;
           PORTAbits.RA7 = 1;
    break; 
    case 3: TRISAbits.TRISA7 = 1;
            TRISCbits.TRISC6 = 0;
            PORTCbits.RC6=0;
    break;
}

}//

void main(void) {
   
   OSCCON = 0x78;  
   OSCTUNE = 0x00; 
   SPLLEN = 1;   

    PORTA = 0;
    PORTB = 0;
    PORTC = 0;
    
    LATA = 0;    
    LATB = 0;    
    LATC = 0;    
   
    TRISE = 0x08;
    TRISA = 0xFF;
    TRISB = 0x2E; //RB0 CCP1 repro, RB4 enable RMS, RB6 RC  
    TRISC = 0x38;  //RC3 a RC4 I2C  RC5 tlatko
    SLRCONC = 0; //max slewrate
    ANSELB = 0x26;  // b1, b2 b3 analog OP   B4 enable RMS, RB5 in z RMS RB6 RC on/off
    ANSELA = 0x7F;   
    WPUE = 0x08;
    WPUB = 0x0;
    WPUA = 0x0;
    WPUC = 0xFF;
    INLVLC =0;  //typ vstupu RC
    OPTION_REGbits.nWPUEN = 0;
    PORTBbits.RB4 = 1; //dis RMS
    PORTBbits.RB6 = 1;
    FVREN = 1; 
    FVRCON= 0x8B;
    APFCON = 0x00;   //alternate pin functio
    CCP1SEL = 1; //CCP1 na RB 0
    CCP2SEL = 1; 
    OPA2CON = 0xC3; 
    OPA1CON = 0xC0;     
    ADCON0 = 0x0; //AN0  
    ADCON1 = 0xE7; //0xE3; // poten nastaven
    ADCON2 = 0x0F; //
    DACOE1 = 0;
    DACOE2 = 0;
  
    PR2 = 249;  
    T2CON = 0x7E; //x6; 
    CCP1CON = 0;
    //CCPR1L = 0x80;     
    INTE = 0;
    GIE = 0;
    
  Delay(300);

   SSPADD = 0x17; //150kHz   
   PORTC &= 0xF8; // RC0, RC1, RC2 = 0;  
   PORTC |=5;
   OpenI2C(8,0);


   TMR1ON = 0;
   ADCON0 = 0x00;  //AN2 RA2 VREF- 
   Delay(3000);
   Leds(0);

   Init();
   StopI2C();
   Delay(200);
   
   TMR2IE = 0;
   TMR1IE = 0;
 
   PEIE = 1;              
   GIE = 1;   
   Delay(50); 
   
   menu = 4;
   
   //main loop  
while(1){
       
/*   mpx=Rng[rng][0]; vmx=Rng[rng][1];   
   port = PORTC;
   port &= 0xF8; 
   port |= mpx; 
   PORTC = port;
  
  if(menu==3  ){      
       vmr=Irng[irng][0]; CCP2CON = 0x0F | ((Irng[irng][1] & 3)<<4); CCPR2L = Irng[irng][1]>>2;
       Delay(20);
  }*/
   
  
   
if( (fre && PORTCbits.RC5==0) || start > 0) 
{ tlc = 1; fre = 0; 
   menu ++; if(menu>5)menu = 0;

  Beep(0);
   CCP1CON = 0;
   CCP2CON = 0;
      
   port = PORTC;
   port &= 0xF8;
   port |= 5; //mpx = 5
   PORTC = port;
   FVRCON = 0x8B;
   PORTBbits.RB4 = 1; //RMS off
   PORTBbits.RB6 = 1; //RC on 
   
   TRISBbits.TRISB3 = 1; 
   PORTBbits.RB0 = 0;
   ADNREF = 1;
   ADCON0 = 0;
   rng = 1;
 
   
switch (menu) {
    case 0:                                                              
        Delay(100);
        Dac(512); // kompenzace nuly
        offs =  nap;
    break;

    case 1:                           
        port= PORTB;
        port&=0xAF;  //RB4, RB6 = 0
        PORTB = port;
        ADCON0 = 0x34;   //RMS               
        Delay(1000);
        Dac(512); // kompenzace nuly
        offs = nap;  
        rng=1;
    break;

    case 2: // dioda                                         
        Delay(100);
        Dac(512);  // kompenzace nuly
        offs = nap; 
        rng=2;
        CCP2CON = 0x0F; CCPR2L =21; 
        TRISBbits.TRISB3 = 0;
    break;

    case 3:  //kiloohm                                         
        Delay(200);
        Dac(512);  // kompenzace nuly
        offs = nap;   
        rng=1; // 2V
        CCP2CON = 0x0F; CCPR2L = 21; 
        TRISBbits.TRISB3 = 0;
        irng=0;
      
    break;

    case 4:     //logic                                       
        Delay(100);
        Dac(512); // kompenzace nuly
        offs = nap;                 
        rng=3; 
        CCP2CON = 0x0F; CCPR2L = 239; 
        TRISBbits.TRISB3 = 0;        
    break;

    case 5://bat RA3                                                              
        ADNREF = 0;                   
        ADCON0 = 0x30;
        Delay(100);
        Dac(512);  // kompenzace nuly
        offs = nap;
        ADCON0 = 0x0C;
    break;
}//switch
   
   } //tlac
 
   Delay(2);
     
   if(PORTCbits.RC5 == 1){fre = 1;  tlc = 0;} 
   

   //mpx=Rng[rng][0]; vmx=Rng[rng][1];
   led = 0; 
    switch (menu) {
            case 0: //SS
                Dac(510); //nastaven rozsahu ss
                if(vmx == 0) nap -= offs - 11*512*2; 
                else nap -= offs - 11*512/vmx; //kompenzace offsetu
                volt = nap / -512;   
                avolt = abs(volt);
                if ((avolt > 4000) && (rng < 8)) rng++;
                if ((avolt < 2000) && (rng > 1))rng--;
                if (rng == 8){over = 1; rng--; Beep(1); led=3; } else Beep(0);
                if(vmx==0)volt = volt/2;                
                else volt = volt * vmx; 
                bgr = 0; //(avolt * vmx) / 1000;
                break;
                
            case 1: //RMS             
                Delay(100);
                Dac(510); //nastaven rozsahu st
                nap -= 10*512;  //kompenzace offsetu
                volt = nap / -256;   // s R40 
                avolt = abs(volt);
                if ((avolt > 2000) && (rng < 8)) rng++;
                if ((avolt < 1000) && (rng > 1))rng--;
                if (rng == 8){over = 1; rng--; Beep(1); led = 3; } else Beep(0);
                
                if(vmx==0)volt = -volt/2;
                else 
                volt = -volt * vmx;
                bgr = 0; //volt / 1000;
                break;
                
            case 2: // diode
                Dac(510);    //nastaven rozsahu            
                rng=2; 
                nap -= offs - 12*512;  //kompenzace offsetu
                volt = nap / -512;   
                avolt = abs(volt);
                if ((avolt > 2200)) {over = 1; bgr = 0; }
                else {over = 0; bgr = volt / 20; }
                if(avolt < 10){Beep(2); led = 1; }else Beep(0);            
                break;

            case 3: //kiloohm
                Dac(510); //nastaven rozsahu  
                rng=1;
                nap -= offs - 25*512; //kompenzace offsetu
                volt = nap / -512;  
                avolt = abs(volt);
                if ((avolt > 3600) && (irng < 7)) irng++;
                if ((avolt <1800) && (irng > 0))irng--;
                if (irng == 7){over = 1; irng--; } // else Beep(0);           
                volt = volt * vmr;   
                bgr = 0; //volt / 100;
                break;

            case 4:   //logic 
                Dac(510); //nastaven rozsahu  
                nap -= offs  - 6*512; //kompenzace offsetu
                volt = nap / -512;  
                avolt = abs(volt);
                if(avolt < 400){over = 0; Beep(3); led = 1;}
                else if(avolt > 1400){over = 0; Beep(1);led = 2; }
                else {Beep(0); over = 1;}                
                volt = volt*2; 
                bgr = volt / 50;
                break;

            case 5:  //bat
                Dac(510); //nastaven rozsahu  
                nap -= offs;
                volt = nap / 512;             
                bgr = 0;
                break;

        }//switch
  
    
   mpx=Rng[rng][0]; vmx=Rng[rng][1];   
   port = PORTC;
   port &= 0xF8; 
   port |= mpx; 
   PORTC = port;
  
  if(menu==3  ){      
       vmr=Irng[irng][0]; CCP2CON = 0x0F | ((Irng[irng][1] & 3)<<4); CCPR2L = Irng[irng][1]>>2;
       Delay(20);
  } 
    
    
   
  if(over == 0){ 
   ltoa(strt, volt, 10);
   nap=0;
   strcpy(str," 0.000");
  len = strlen((const char *)strt);
  if(strt[0]=='-'){str[0]='-'; len--; j=1;}
   else {str[0]=' '; j=0;  }
  
  if(len>5){str[1] = strt[j];
   str[2]= strt[(char)(1+j)];
   str[3]= strt[(char)(2+j)];
   str[4] ='.';
   str[5]= strt[(char)(3+j)];
   str[6] = ' ';
   }
  
  else if(len>4){str[1] = strt[j];
   str[2]= strt[(char)(1+j)];
   str[3] ='.';
   str[4]= strt[(char)(2+j)];
   str[5]= strt[(char)(3+j)];
   str[6] = ' ';
   }
   
  else if(len>3){str[1] = strt[j];
   str[3]= strt[(char)(1+j)];
   str[4]= strt[(char)(2+j)];
   str[5]= strt[(char)(3+j)];
   }
  
   else if(len>2){str[3] = strt[j];
   str[4]= strt[(char)(1+j)];
   str[5]= strt[(char)(2+j)];
   }
   
   else if(len>1){str[4] = strt[j];
   str[5] = strt[(char)(1+j)];
           
   } 
   else str[5] = strt[j];
 
  }
  else {strcpy(str," ---- "); over = 0;
  
  }
  
  Leds(led); 
  HorizontalGraph2( 0, bgr); 
    SetCursor(0,0);
    StringBig(str);
    
    SetCursor(0,110);
   BigDisp(menu+11+0x30);
 

   
StopI2C();
//Delay(1);
if(start>0){start--; if(start==1)Delay(3000); }
     };
}//


void interrupt ISBR() {
    char alt = 0;
   
   if (TMR2IF) {
        TMR2IF = 0;
       alt = PORTB;  
       alt ^= 1;
       PORTB = alt;   
   }
 };

